/* Custom styles for AsyncMCP documentation */

/* Reference: Example of light mode code styling (commented out)
[data-theme="light"] pre code {
  background-color: #f6f8fa !important;
  color: #24292e !important;
}
*/